@@ -10,27 +10,33 @@ from picasso import postprocess
 from functools import partial
 import concurrent.futures
 import multiprocessing
-from shapely.geometry import Point
 from multiprocessing import Manager
 import numpy as np
 import pandas as pd
 import math
 import cv2
 from skimage.feature import peak_local_max
-
+from shapely.geometry import Polygon, Point, MultiPolygon, MultiPoint
+from shapely.strtree import STRtree
 
 def precompute_kernels(lnoise=0, lobject=1):
 
+    # Determine the kernel size based on the larger of the two scales
+    size = 2 * max(math.ceil(5 * lnoise), round(lobject)) + 1
+
+    # Handling Gaussian Kernel
     if lnoise == 0:
-        gaussian_kernel = 1
+        gaussian_kernel = np.array([[1]])
     else:
-        gaussian_kernel = cv2.getGaussianKernel(2 * math.ceil(5 * lnoise) + 1, math.sqrt(2) * lnoise)
+        gaussian_kernel_1d = cv2.getGaussianKernel(size, math.sqrt(2) * lnoise)
+        gaussian_kernel = np.outer(gaussian_kernel_1d, gaussian_kernel_1d)
 
     gaussian_kernel_convolved = cv2.filter2D(gaussian_kernel, -1, gaussian_kernel)
 
+    # Handling Boxcar Kernel
     if lobject != 0:
-        boxcar_kernel = np.ones(2 * round(lobject) + 1)
-        boxcar_kernel = boxcar_kernel / sum(boxcar_kernel)
+        boxcar_kernel = np.ones((size, size))
+        boxcar_kernel = boxcar_kernel / np.sum(boxcar_kernel)
         boxcar_kernel_convolved = cv2.filter2D(boxcar_kernel, -1, boxcar_kernel)
     else:
         boxcar_kernel_convolved = None
@@ -70,6 +76,7 @@ def detect_moltrack_locs(dat, progress_list, fit_list):
         box_size = dat["box_size"]
         roi = dat["roi"]
         dataset = dat["dataset"]
+        channel = dat["channel"]
         start_index = dat["start_index"]
         end_index = dat["end_index"]
         detect = dat["detect"]
@@ -80,6 +87,12 @@ def detect_moltrack_locs(dat, progress_list, fit_list):
         polygons = dat["polygons"]
         threshold = dat["threshold"]
         window_size = dat["window_size"]
+        segmentation_layer = dat["segmentation_layer"]
+
+        if segmentation_layer not in ["None",""]:
+            seg_name = segmentation_layer[:-1].lower() + "_index"
+        else:
+            seg_name = "shape_index"
 
         loc_list = []
         spot_list = []
@@ -103,7 +116,8 @@ def detect_moltrack_locs(dat, progress_list, fit_list):
 
                     filtered_frame = bandpass(frame.copy(),kernels)
 
-                    locs = peak_local_max(filtered_frame, min_distance=1, threshold_abs=threshold)
+                    locs = peak_local_max(filtered_frame, min_distance=1,
+                        threshold_abs=threshold)
 
                     locs = pd.DataFrame(locs, columns=["y", "x"])
                     locs.insert(0, "frame", 0)
@@ -112,8 +126,11 @@ def detect_moltrack_locs(dat, progress_list, fit_list):
                     if remove_overlapping:
                         locs = remove_overlapping_locs(locs, box_size)
 
+                    polygon_indices = get_polygon_indices(polygons, locs)
+
                     if polygon_filter:
-                        locs = remove_segmentation_locs(locs, polygons)
+                        locs, polygon_indices = remove_segmentation_locs(polygons,
+                            locs, polygon_indices)
 
                     if len(locs) > 0:
 
@@ -125,6 +142,11 @@ def detect_moltrack_locs(dat, progress_list, fit_list):
 
                         locs = pd.DataFrame(locs)
                         locs.insert(0, "dataset", dataset)
+                        locs.insert(1, "channel", channel)
+
+                        if segmentation_layer not in ["None",""]:
+                            locs[seg_name] = polygon_indices
+
                         locs = locs.to_records(index=False)
 
                         for loc, spot in zip(locs, spot_data):
@@ -260,31 +282,60 @@ def fit_spots_lq(spots, locs, box, progress_list):
 
     return locs
 
-def remove_segmentation_locs(locs, polygons):
 
-    if len(polygons) > 0 and len(locs) > 0:
+def get_polygon_indices(polygons, locs):
 
-        loclist = pd.DataFrame(locs).to_dict(orient="records")
+    polygon_indices = [-1] * len(locs)
 
-        filtered_locs = []
+    try:
 
-        for loc in loclist:
-            point = Point(loc["x"], loc["y"])
+        if len(polygons) > 0 and len(locs) > 0:
+
+            coords = np.stack([locs["x"], locs["y"]], axis=1)
+            points = [Point(coord) for coord in coords]
+
+            spatial_index = STRtree(points)
 
             for polygon_index, polygon in enumerate(polygons):
-                if polygon.contains(point):
-                    loc["segmentation"] = polygon_index
-                    filtered_locs.append(loc)
 
-        if len(filtered_locs):
-            locs = pd.DataFrame(filtered_locs).to_records(index=False)
-        else:
-            locs = []
+                possible_points = spatial_index.query(polygon)
 
-    else:
-        locs = []
+                for point_index in possible_points:
 
-    return locs
+                    point = points[point_index]
+
+                    if polygon.contains(point):
+
+                        polygon_indices[point_index] = polygon_index
+
+    except:
+        print(traceback.format_exc())
+        pass
+
+    return polygon_indices
+
+
+
+def remove_segmentation_locs(polygons, locs, polygon_indices):
+
+    try:
+
+        if len(polygon_indices) > 0:
+
+            delete_indices = np.argwhere(np.array(polygon_indices) == -1).flatten()
+
+            polygon_indices = np.delete(np.array(polygon_indices), delete_indices)
+
+            mask = np.ones(len(locs), dtype=bool)
+            mask[delete_indices] = False
+
+            locs = locs[mask]
+
+    except:
+        print(traceback.format_exc())
+        pass
+
+    return locs, polygon_indices
 
 def detect_picaso_locs(dat, progress_list, fit_list):
 
@@ -295,6 +346,7 @@ def detect_picaso_locs(dat, progress_list, fit_list):
         box_size = dat["box_size"]
         roi = dat["roi"]
         dataset = dat["dataset"]
+        channel = dat["channel"]
         start_index = dat["start_index"]
         end_index = dat["end_index"]
         detect = dat["detect"]
@@ -325,8 +377,11 @@ def detect_picaso_locs(dat, progress_list, fit_list):
                 if remove_overlapping:
                     locs = remove_overlapping_locs(locs, box_size)
 
+                polygon_indices = get_polygon_indices(polygons, locs)
+
                 if polygon_filter:
-                    locs = remove_segmentation_locs(locs, polygons)
+                    locs, polygon_indices = remove_segmentation_locs(polygons,
+                        locs, polygon_indices)
 
                 if len(locs) > 0:
 
@@ -342,6 +397,7 @@ def detect_picaso_locs(dat, progress_list, fit_list):
 
                         #instert dataset at column 0
                         locs.insert(0, "dataset", dataset)
+                        locs.insert(1, "channel", channel)
 
                         locs = locs.to_records(index=False)
 
@@ -365,48 +421,6 @@ def detect_picaso_locs(dat, progress_list, fit_list):
 
 class _picasso_detect_utils:
 
-    def populate_localisation_dict(self, loc_dict, render_loc_dict, detect_mode,
-            image_channel, box_size, fitted=False):
-
-        if self.verbose:
-            print("Populating localisation dictionary...")
-
-        detect_mode = detect_mode.lower()
-
-        try:
-
-            for dataset_name, locs in loc_dict.items():
-
-                if detect_mode == "localisations":
-
-                    if dataset_name not in self.localisation_dict["localisations"].keys():
-                        self.localisation_dict["localisations"][dataset_name] = {}
-                    if image_channel not in self.localisation_dict["localisations"][dataset_name].keys():
-                        self.localisation_dict["localisations"][dataset_name][image_channel.lower()] = {}
-
-
-                    fiducial_dict = {"localisations": []}
-
-                    fiducial_dict["localisations"] = locs.copy()
-                    fiducial_dict["fitted"] = fitted
-                    fiducial_dict["box_size"] = box_size
-
-                    self.localisation_dict["localisations"][dataset_name][image_channel.lower()] = fiducial_dict.copy()
-
-                else:
-
-                    self.localisation_dict["bounding_boxes"]["localisations"] = locs.copy()
-                    self.localisation_dict["bounding_boxes"]["fitted"] = fitted
-                    self.localisation_dict["bounding_boxes"]["box_size"] = box_size
-
-
-        except:
-            print(traceback.format_exc())
-            self.gui.picasso_progressbar.setValue(0)
-            self.gui.picasso_detect.setEnabled(True)
-            self.gui.picasso_fit.setEnabled(True)
-            self.gui.picasso_detectfit.setEnabled(True)
-
     def _picasso_wrapper_finished(self):
 
         try:
@@ -471,7 +485,7 @@ class _picasso_detect_utils:
             print(traceback.format_exc())
             return None
 
-    def get_fit_data(self, dataset_list, box_size, frame_index=None):
+    def get_fit_data(self, dataset_list, channel_list, box_size, frame_index=None):
 
         try:
 
@@ -482,31 +496,36 @@ class _picasso_detect_utils:
 
                 if dataset in self.localisation_dict.keys():
 
-                    loc_dict = self.localisation_dict[dataset]
+                    for channel in channel_list:
 
-                    locs = loc_dict["localisations"].copy()
+                        if channel in self.localisation_dict[dataset].keys():
 
-                    if frame_index is not None:
-                        locs = locs[locs.frame == frame_index]
+                            loc_dict = self.localisation_dict[dataset][channel]
 
-                    if len(locs) > 0:
+                            locs = loc_dict["localisations"].copy()
 
-                        image_dict = self.dataset_dict[dataset]
+                            if frame_index is not None:
+                                locs = locs[locs.frame == frame_index]
 
-                        if "dataset" not in locs.dtype.names:
-                            locs = pd.DataFrame(locs)
-                            locs.insert(0, "dataset", dataset)
-                            locs = locs.to_records(index=False)
+                            if len(locs) > 0:
 
-                        image = image_dict.pop("data")
+                                image_dict = self.dataset_dict[dataset]["images"]
 
-                        camera_info = {"baseline": 100.0, "gain": 1, "sensitivity": 1.0, "qe": 0.9, }
-                        spot_data = get_spots(image, locs, box_size, camera_info)
+                                if "dataset" not in locs.dtype.names:
+                                    locs = pd.DataFrame(locs)
+                                    locs.insert(0, "dataset", dataset)
+                                    locs.insert(1, "channel", channel)
+                                    locs = locs.to_records(index=False)
 
-                        loc_list.append(locs)
-                        spot_list.append(spot_data)
+                                image = image_dict.pop(channel)
 
-                        image_dict["data"] = image
+                                camera_info = {"baseline": 100.0, "gain": 1, "sensitivity": 1.0, "qe": 0.9, }
+                                spot_data = get_spots(image, locs, box_size, camera_info)
+
+                                loc_list.append(locs)
+                                spot_list.append(spot_data)
+
+                                image_dict[channel] = image
 
         except:
             print(traceback.format_exc())
@@ -521,7 +540,7 @@ class _picasso_detect_utils:
 
 
     def populate_picasso_detect_jobs(self, detect, fit,
-            min_net_gradient, roi):
+            min_net_gradient, roi_dict):
 
         try:
 
@@ -530,11 +549,12 @@ class _picasso_detect_utils:
 
             box_size = int(self.gui.picasso_box_size.currentText())
             remove_overlapping = self.gui.picasso_remove_overlapping.isChecked()
-            polygon_filter = self.gui.picasso_segmentation_filtering.isChecked()
+            segmentation_layer = self.gui.picasso_segmentation_layer.currentText()
+            polygon_filter = self.gui.picasso_segmentation_filter.isChecked()
             threshold = int(self.gui.moltrack_threshold.text())
             window_size = int(self.gui.moltrack_window_size.text())
 
-            segmentation_polygons = self.get_segmentation_polygons()
+            segmentation_polygons = self.get_segmentation_polygons(segmentation_layer)
 
             compute_jobs = []
 
@@ -543,7 +563,11 @@ class _picasso_detect_utils:
 
             for image_chunk in self.shared_chunks:
 
+                dataset = image_chunk["dataset"]
+                channel = image_chunk["channel"]
+
                 compute_job = {"dataset": image_chunk["dataset"],
+                               "channel": image_chunk["channel"],
                                "start_index": image_chunk["start_index"],
                                "end_index": image_chunk["end_index"],
                                "shared_memory_name": image_chunk["shared_memory_name"],
@@ -555,8 +579,9 @@ class _picasso_detect_utils:
                                "box_size": int(box_size),
                                "threshold": threshold,
                                "window_size": window_size,
-                               "roi": roi,
+                               "roi": roi_dict[dataset][channel],
                                "remove_overlapping": remove_overlapping,
+                               "segmentation_layer": segmentation_layer,
                                "polygon_filter":polygon_filter,
                                "polygons": segmentation_polygons,
                                "stop_event": self.stop_event, }
@@ -710,7 +735,8 @@ class _picasso_detect_utils:
 
 
     def _picasso_wrapper(self, progress_callback, detect, fit,
-            min_net_gradient, dataset_list = [], frame_index = None, detect_mode = "Picasso", fit_mode = "Picasso"):
+            min_net_gradient, dataset_list = [], channel_list = [],
+            frame_index = None, detect_mode = "Picasso", fit_mode = "Picasso"):
 
         try:
             locs, fitted = [], False
@@ -718,7 +744,7 @@ class _picasso_detect_utils:
             frame_mode = self.gui.picasso_frame_mode.currentText()
             detect_mode = self.gui.smlm_detect_mode.currentText()
             box_size = int(self.gui.picasso_box_size.currentText())
-            roi = self.generate_roi()
+            roi_dict = self.generate_roi()
 
             if frame_mode.lower() == "active":
                 executor_class = concurrent.futures.ThreadPoolExecutor
@@ -734,10 +760,11 @@ class _picasso_detect_utils:
                     if detect is True:
 
                         self.create_shared_image_chunks(dataset_list=dataset_list,
-                            frame_index=frame_index, chunk_size = 1000)
+                            channel_list=channel_list, frame_index=frame_index,
+                            chunk_size = 1000)
 
                         detect_jobs, n_frames = self.populate_picasso_detect_jobs(detect,
-                            fit, min_net_gradient, roi)
+                            fit, min_net_gradient, roi_dict)
 
                         print(f"Starting Picasso {len(detect_jobs)} compute jobs...")
 
@@ -754,7 +781,8 @@ class _picasso_detect_utils:
 
                     if detect is False and fit is True:
 
-                        locs, spots = self.get_fit_data(dataset_list, box_size, frame_index)
+                        locs, spots = self.get_fit_data(dataset_list, channel_list,
+                            box_size, frame_index)
 
                     if len(locs) > 0 and fit == True:
 
@@ -802,37 +830,52 @@ class _picasso_detect_utils:
             if len(locs) > 0:
 
                 dataset_list = list(set(locs["dataset"]))
+                channel_list = list(set(locs["channel"]))
 
                 for dataset in dataset_list:
 
                     if dataset not in self.localisation_dict.keys():
                         self.localisation_dict[dataset] = {}
 
-                    result_dict = self.localisation_dict[dataset]
-
                     dataset_locs = locs[locs["dataset"] == dataset]
 
-                    if len(dataset_locs) == 0:
+                    for channel in channel_list:
 
-                        result_dict["localisations"] = []
-                        result_dict["fitted"] = False
-                        result_dict["box_size"] = box_size
+                        channel_locs = dataset_locs[dataset_locs["channel"] == channel]
 
-                    else:
+                        if "lpx" in channel_locs.dtype.names:
+                            channel_locs = channel_locs[~np.isnan(channel_locs["lpx"])]
+                            channel_locs = channel_locs[~np.isnan(channel_locs["lpy"])]
 
-                        loc_cols = list(dataset_locs.dtype.names)
+                        if channel not in self.localisation_dict[dataset].keys():
+                            self.localisation_dict[dataset][channel] = {}
 
-                        if "dataset" in loc_cols or "channel" in loc_cols:
-                            dataset_locs = pd.DataFrame(dataset_locs)
+                        result_dict = self.localisation_dict[dataset][channel]
 
-                            if "dataset" in loc_cols:
-                                dataset_locs = dataset_locs.drop(columns=["dataset"])
+                        if len(channel_locs) == 0:
 
-                            dataset_locs = dataset_locs.to_records(index=False)
+                            result_dict["localisations"] = []
+                            result_dict["fitted"] = False
+                            result_dict["box_size"] = box_size
 
-                        result_dict["localisations"] = dataset_locs.copy()
-                        result_dict["fitted"] = fitted
-                        result_dict["box_size"] = box_size
+                        else:
+
+                            loc_cols = list(channel_locs.dtype.names)
+
+                            if "dataset" in loc_cols or "channel" in loc_cols:
+                                channel_locs = pd.DataFrame(channel_locs)
+
+                                if "dataset" in loc_cols:
+                                    channel_locs = channel_locs.drop(columns=["dataset"])
+
+                                if "channel" in loc_cols:
+                                    channel_locs = channel_locs.drop(columns=["channel"])
+
+                                channel_locs = channel_locs.to_records(index=False)
+
+                            result_dict["localisations"] = channel_locs.copy()
+                            result_dict["fitted"] = fitted
+                            result_dict["box_size"] = box_size
 
         except:
             print(traceback.format_exc())
@@ -849,6 +892,7 @@ class _picasso_detect_utils:
                 detect_mode = self.gui.smlm_detect_mode.currentText()
                 fit_mode = self.gui.smlm_fit_mode.currentText()
                 dataset_name = self.gui.picasso_dataset.currentText()
+                channel_name = self.gui.picasso_channel.currentText()
                 min_net_gradient = self.gui.picasso_min_net_gradient.text()
                 frame_mode = self.gui.picasso_frame_mode.currentText()
                 minimise_ram = self.gui.picasso_minimise_ram.isChecked()
@@ -876,10 +920,29 @@ class _picasso_detect_utils:
                     else:
                         dataset_list = [dataset_name]
 
+                    if channel_name == "All Channels":
+
+                        channel_list = []
+                        for dataset_name in self.dataset_dict.keys():
+                            try:
+                                image_dict = self.dataset_dict[dataset_name]["images"]
+                                channel_list.append(set(image_dict.keys()))
+                            except:
+                                pass
+
+                        channel_list = set.intersection(*channel_list)
+                        channel_list = list(channel_list)
+
+                    else:
+                        channel_list = [channel_name]
+
+
+
                     self.worker = Worker(self._picasso_wrapper,
                         detect=detect, fit=fit,
                         min_net_gradient=min_net_gradient,
                         dataset_list=dataset_list,
+                        channel_list=channel_list,
                         detect_mode=detect_mode,
                         fit_mode=fit_mode,
                         frame_index=frame_index)
@@ -904,7 +967,7 @@ class _picasso_detect_utils:
         border_width = self.gui.picasso_roi_border_width.text()
         window_cropping = self.gui.picasso_window_cropping .isChecked()
 
-        roi = None
+        roi_dict = {}
 
         try:
 
@@ -931,68 +994,56 @@ class _picasso_detect_utils:
             if generate_roi:
 
                 dataset = self.gui.picasso_dataset.currentText()
+                channel = self.gui.picasso_channel.currentText()
 
                 if dataset == "All Datasets":
-                    dataset = list(self.dataset_dict.keys())[0]
-
-                image_shape = self.dataset_dict[dataset]["data"].shape
-
-                frame_shape = image_shape[1:]
-
-                if window_cropping:
-
-                    border_width = int(border_width)
-
-                    if x1 < border_width:
-                        x1 = border_width
-                    if y1 < border_width:
-                        y1 = border_width
-                    if x2 > frame_shape[1] - border_width:
-                        x2 = frame_shape[1] - border_width
-                    if y2 > frame_shape[0] - border_width:
-                        y2 = frame_shape[0] - border_width
-
-                    roi = [[y1, x1], [y2, x2]]
-
+                    dataset_list = list(self.dataset_dict.keys())
                 else:
+                    dataset_list = [dataset]
 
-                    roi = [[int(border_width), int(border_width)],
-                           [int(frame_shape[0] - border_width), int(frame_shape[1] - border_width)]]
+                for dataset_name in dataset_list:
+
+                    if channel == "All Channels":
+                        channel_list = list(self.dataset_dict[dataset_name]["images"].keys())
+                    else:
+                        channel_list = [channel]
+
+                    for channel_name in channel_list:
+
+                        if dataset_name not in roi_dict.keys():
+                            roi_dict[dataset_name] = {}
+                        if channel_name not in roi_dict[dataset_name].keys():
+                            roi_dict[dataset_name][channel_name] = {}
+
+                        image_dict = self.dataset_dict[dataset_name]["images"]
+                        image_shape = image_dict[channel_name].shape
+
+                        frame_shape = image_shape[1:]
+
+                        if window_cropping:
+
+                            border_width = int(border_width)
+
+                            if x1 < border_width:
+                                x1 = border_width
+                            if y1 < border_width:
+                                y1 = border_width
+                            if x2 > frame_shape[1] - border_width:
+                                x2 = frame_shape[1] - border_width
+                            if y2 > frame_shape[0] - border_width:
+                                y2 = frame_shape[0] - border_width
+
+                            roi = [[y1, x1], [y2, x2]]
+
+                        else:
+
+                            roi = [[int(border_width), int(border_width)],
+                                   [int(frame_shape[0] - border_width), int(frame_shape[1] - border_width)]]
+
+                        roi_dict[dataset_name][channel_name] = roi
 
         except:
             print(traceback.format_exc())
             pass
 
-        return roi
-
-    def export_picasso_locs(self, locs):
-
-        if self.verbos:
-            print("Exporting Picasso locs")
-
-        try:
-
-            dataset_name = self.gui.picasso_dataset.currentText()
-            image_channel = self.gui.picasso_channel.currentText()
-            min_net_gradient = int(self.gui.picasso_min_net_gradient.text())
-            box_size = int(self.gui.picasso_box_size.currentText())
-
-            path = self.dataset_dict[dataset_name][image_channel.lower()]["path"]
-            image_shape = self.dataset_dict[dataset_name][image_channel.lower()]["data"].shape
-
-            base, ext = os.path.splitext(path)
-            path = base + f"_{image_channel}_picasso_locs.hdf5"
-
-            info = [{"Byte Order": "<", "Data Type": "uint16", "File": path,
-                     "Frames": image_shape[0], "Height": image_shape[1],
-                     "Micro-Manager Acquisiton Comments": "", "Width": image_shape[2], },
-                    {"Box Size": box_size, "Fit method": "LQ, Gaussian",
-                     "Generated by": "Picasso Localize",
-                     "Min. Net Gradient": min_net_gradient, "Pixelsize": 130, "ROI": None, }]
-
-            from picasso.io import save_locs
-            # save_locs(path, locs, info)
-
-        except:
-            print(traceback.format_exc())
-            pass
+        return roi_dict
