@@ -72,7 +72,7 @@ class _segmentation_utils:
 
         gpu = False
 
-        if torch.cuda.is_available() and self.cellpose_usegpu.isChecked():
+        if torch.cuda.is_available():
             if self.widget_notifications:
                 show_info("Cellpose Using GPU")
             gpu = True
@@ -101,7 +101,7 @@ class _segmentation_utils:
 
         gpu = False
 
-        if torch.cuda.is_available() and self.cellpose_usegpu.isChecked():
+        if torch.cuda.is_available():
             if self.widget_notifications:
                 show_info("Omnipose Using GPU")
             gpu = True
@@ -192,24 +192,33 @@ class _segmentation_utils:
         if self.dataset_dict != {} or hasattr(self, "segmentation_image"):
 
             dataset = self.gui.cellpose_dataset.currentText()
+            channel = self.gui.cellpose_channel.currentText()
             current_frame = self.viewer.dims.current_step[0]
 
             if dataset == "Segmentation Image":
                 if hasattr(self, "segmentation_image"):
-                    images = self.segmentation_image.copy()
+                    data = self.segmentation_image.copy()
 
                     if mode == "active":
-                        images = [images[current_frame]]
+                        if len(data.shape) == 3:
+                            if data.shape[0] > 1:
+                                if current_frame < data.shape[0]:
+                                    images = [data[current_frame]]
+                            else:
+                                images = [data[0]]
                     else:
-                        if len(images.shape) == 3:
-                            images = [frame for frame in images]
+                        if len(data.shape) == 3:
+                            images = [frame for frame in data]
                         else:
-                            images = [images]
+                            images = [data]
             else:
+                data = self.dataset_dict[dataset]["images"][channel]
                 if mode == "active":
-                    images = [self.dataset_dict[dataset]["data"][current_frame]]
+                    if len(data.shape) == 3:
+                        if current_frame < data.shape[0]:
+                            images = [data[current_frame]]
                 else:
-                    images = self.dataset_dict[dataset]["data"].copy()
+                    images = data.copy()
 
                     if len(images.shape) == 3:
                         images = [frame for frame in images]
@@ -259,7 +268,14 @@ class _segmentation_utils:
 
             if len(shapes) > 0:
 
-                self.initialise_segLayer(shapes)
+                dataset = self.gui.cellpose_dataset.currentText()
+
+                if dataset == "Segmentation Image":
+                    pixel_size = self.segmentation_image_pixel_size
+                else:
+                    pixel_size = self.dataset_dict[dataset]["pixel_size"]
+
+                self.initialise_segLayer(shapes, pixel_size)
 
 
 
@@ -433,50 +449,35 @@ class _segmentation_utils:
 
         return post_processed_mask
 
-    def get_segmentation_polygons(self):
+    def get_segmentation_polygons(self, segmentation_layer = None):
+
+        polygons = []
 
-        filter = self.gui.picasso_segmentation_filtering.isChecked()
         layer_names = [layer.name for layer in self.viewer.layers]
 
-        if "Segmentations" in layer_names and filter:
+        if segmentation_layer in layer_names and filter:
 
-            segmentations = self.viewer.layers["Segmentations"].data.copy()
+            shapes = self.viewer.layers[segmentation_layer].data.copy()
+            shape_types = self.viewer.layers[segmentation_layer].shape_type
 
-            if len(segmentations) > 0:
+            for seg, seg_type in zip(shapes, shape_types):
+                if seg_type == "polygon":
+                    ndim = seg.shape[1]
 
-                ndim = segmentations[0].shape[1]
+                    if ndim == 2:
 
-                if ndim == 2:
-
-                    polygons = []
-
-                    for seg in segmentations:
                         seg = np.fliplr(seg)
                         poly = Polygon(seg)
                         polygons.append(poly)
 
-                    return polygons
-
-                elif ndim == 3:
-
-                    polygons = {}
-
-                    for seg in segmentations:
-
-                        frame = int(seg[0, 0])
-                        if frame not in polygons.keys():
-                            polygons[frame] = []
+                    elif ndim == 3:
 
                         seg = seg[:, 1:]
                         seg = np.fliplr(seg)
                         poly = Polygon(seg)
+                        polygons.append(poly)
 
-                        polygons[frame].append(poly)
-
-                    return polygons
-
-        return []
-
+        return polygons
 
     def dilate_segmentations(self, viewer = None, simplify = True):
 
